@@ -225,19 +225,47 @@
   (should (equal-including-properties
            (parsebib-clean-TeX-markup "The verb \\textit{krijgen} as an undative verb.")
            #("The verb krijgen as an undative verb." 9 16
-             (face italic)))))
+             (face italic))))
+  (should (equal
+           (let ((parsebib-TeX-cleanup-target 'markdown))
+             (parsebib-clean-TeX-markup "The verb \\textit{krijgen} as an undative verb."))
+           "The verb *krijgen* as an undative verb."))
+  (should (equal
+           (let ((parsebib-TeX-cleanup-target 'org))
+             (parsebib-clean-TeX-markup "The verb \\textit{krijgen} as an undative verb."))
+           "The verb /krijgen/ as an undative verb."))
+  (should (equal
+           (let ((parsebib-TeX-cleanup-target 'plain))
+             (parsebib-clean-TeX-markup "The verb \\textit{krijgen} as an undative verb."))
+           "The verb krijgen as an undative verb.")))
 
 (ert-deftest parsebib-clean-TeX-markup-emph ()
   (should (equal-including-properties
            (parsebib-clean-TeX-markup "The verb \\emph{krijgen} as an undative verb.")
            #("The verb krijgen as an undative verb." 9 16
-             (face italic)))))
+             (face italic))))
+  (should (equal
+           (let ((parsebib-TeX-cleanup-target 'markdown))
+             (parsebib-clean-TeX-markup "The verb \\emph{krijgen} as an undative verb."))
+           "The verb *krijgen* as an undative verb."))
+  (should (equal
+           (let ((parsebib-TeX-cleanup-target 'org))
+             (parsebib-clean-TeX-markup "The verb \\emph{krijgen} as an undative verb."))
+           "The verb /krijgen/ as an undative verb.")))
 
 (ert-deftest parsebib-clean-TeX-markup-textbf ()
   (should (equal-including-properties
            (parsebib-clean-TeX-markup "The verb \\textbf{krijgen} as an undative verb.")
            #("The verb krijgen as an undative verb." 9 16
-             (face bold)))))
+             (face bold))))
+  (should (equal
+           (let ((parsebib-TeX-cleanup-target 'markdown))
+             (parsebib-clean-TeX-markup "The verb \\textbf{krijgen} as an undative verb."))
+           "The verb **krijgen** as an undative verb."))
+  (should (equal
+           (let ((parsebib-TeX-cleanup-target 'org))
+             (parsebib-clean-TeX-markup "The verb \\textbf{krijgen} as an undative verb."))
+           "The verb *krijgen* as an undative verb.")))
 
 (ert-deftest parsebib-clean-TeX-markup-textsc ()
   (should (equal
@@ -420,4 +448,272 @@
                           nil nil #'equal)))
            "/Users/xxxx/Zotero/storage/ZJVUZD8F/Ahn et al. - Identifying On-Off-CPU Bottlenecks Together with Blocked Samples  USENIX.pdf")))
 
+;;; Test inheritance
+(ert-deftest parsebib-test-parse-bib-buffer-biblatex-inheritance ()
+  (should (equal
+           (with-temp-buffer
+             (insert
+              "@InCollection{Ackema:Neeleman2002,\n"
+	      "pages = {219-256},\n"
+	      "title = {Effects of Short-Term Storage in Processing Rightward Movement},\n"
+	      "crossref = {Nooteboom:Weerman:Wijnen},\n"
+	      "author = {Ackema, Peter and Neeleman, Ad}}\n"
+              "\n"
+              "@Collection{Nooteboom:Weerman:Wijnen,\n"
+	      "  year = {2002},\n"
+	      "  publisher = Klw,\n"
+	      "  title = {Storage and Computation in the Language Faculty},\n"
+	      "  editor = {Nooteboom, Sieb and Weerman, Fred and Wijnen, Frank}}\n")
+             (let* ((results (parsebib-parse-bib-buffer :inheritance 'biblatex)))
+               (alist-get "booktitle" (gethash "Ackema:Neeleman2002" (car results))
+                          nil nil #'equal)))
+           "{Storage and Computation in the Language Faculty}")))
+
+;;; Test the RDP
+(ert-deftest parsebib-test-@comment ()
+  (should (equal
+           (with-temp-buffer
+             (insert "@Comment{ -*-coding: utf-8 -*- }\n")
+             (goto-char (point-min))
+             (parsebib--@comment))
+           "{ -*-coding: utf-8 -*- }"))
+  (should (equal
+           (with-temp-buffer
+             (insert "@Comment -*-coding: utf-8 -*-\n")
+             (goto-char (point-min))
+             (parsebib--@comment))
+           "-*-coding: utf-8 -*-"))
+  (should (equal
+           (with-temp-buffer
+             (insert "@Comment{\n"
+                     "    Local Variables:\n"
+                     "    bibtex-dialect: biblatex\n"
+                     "    End:\n"
+                     "}\n")
+             (goto-char (point-min))
+             (parsebib--@comment))
+           (concat "{\n"
+                   "    Local Variables:\n"
+                   "    bibtex-dialect: biblatex\n"
+                   "    End:\n"
+                   "}"))))
+
+(ert-deftest parsebib-test-@string ()
+  ;; @String definition with curly braces.
+  (should (equal
+           (with-temp-buffer
+             (insert "@String{MGrt = {Berlin: Mouton de Gruyter}}")
+             (goto-char (point-min))
+             (parsebib--@string))
+           (list "MGrt" "{Berlin: Mouton de Gruyter}")))
+  ;; @String definition with double quotes.
+  (should (equal
+           (with-temp-buffer
+             (insert "@String{LI = \"Linguistic Inquiry\"}")
+             (goto-char (point-min))
+             (parsebib--@string))
+           (list "LI" "\"Linguistic Inquiry\"")))
+  ;; @String definition with @String abbrev.
+  (should (equal
+           (with-temp-buffer
+             (insert "@String{CUP = {Cambridge: Cambridge } # UP}")
+             (goto-char (point-min))
+             (parsebib--@string))
+           (list "CUP" "{Cambridge: Cambridge }" "UP")))
+  ;; @String definition missing closing curly brace.
+  (should-error (with-temp-buffer
+                  (insert "@String{CUP = {Cambridge: Cambridge } # UP\n")
+                  (goto-char (point-min))
+                  (parsebib--@string))
+                :type 'parsebib-error)
+  ;; @String definition with mismatched brace/parenthesis.
+  (should-error (with-temp-buffer
+                  (insert "@String{CUP = {Cambridge: Cambridge } # UP)\n")
+                  (goto-char (point-min))
+                  (parsebib--@string))
+                :type 'parsebib-error)
+  ;; @String abbreviation without expansion.
+  (should (equal (with-temp-buffer
+                   (insert "@Article{Potapov_2016aa,\n"
+                           "    author = {Potapov, Denis and Sukochev, Fedor and Zanin, Dmitriy},\n"
+                           "    month = dec,\n"
+                           "    title = {{Krein's trace theorem revisited}},\n"
+                           "    url = {http://arxiv.org/abs/1701.00697v1},\n"
+                           "    year = {2016}\n"
+                           "}\n")
+                   (goto-char (point-min))
+                   (let ((results (parsebib-read-entry nil #s(hash-table size 10 data nil test equal))))
+                     (alist-get "month" results nil nil #'equal)))
+                 "dec")))
+
+(ert-deftest parsebib-test-read-entry-nested-braces ()
+  (should (equal
+           (with-temp-buffer
+             (insert "@article{10.1162/coli_a_00528,\n"
+                     "    title = {Usage-based {Grammar Induction} from {Minimal Cognitive Principles}}\n"
+                     "}\n")
+             (goto-char (point-min))
+             (let ((results (parsebib-read-entry)))
+               (alist-get "title" results nil nil #'equal)))
+           "{Usage-based {Grammar Induction} from {Minimal Cognitive Principles}}")))
+
+(ert-deftest parsebib-test-read-entry-after-last-field ()
+  ;; The last field in an entry does not have to have a comma after it:
+  (should (equal
+           (with-temp-buffer
+             (insert "@article{10.1162/coli_a_00528,\n"
+                     "    title = {Usage-based Grammar Induction from Minimal Cognitive Principles}\n"
+                     "}\n")
+             (goto-char (point-min))
+             (let ((results (parsebib-read-entry)))
+               (cons (alist-get "=key=" results nil nil #'equal)
+                     (alist-get "title" results nil nil #'equal))))
+           (cons "10.1162/coli_a_00528"
+                 "{Usage-based Grammar Induction from Minimal Cognitive Principles}")))
+  ;; But there *may* be a comma after the last field:
+  (should (equal
+           (with-temp-buffer
+             (insert "@article{10.1162/coli_a_00528,\n"
+                     "    title = {Usage-based Grammar Induction from Minimal Cognitive Principles},\n"
+                     "}\n")
+             (goto-char (point-min))
+             (let ((results (parsebib-read-entry)))
+               (cons (alist-get "=key=" results nil nil #'equal)
+                     (alist-get "title" results nil nil #'equal))))
+           (cons "10.1162/coli_a_00528"
+                 "{Usage-based Grammar Induction from Minimal Cognitive Principles}"))))
+
+;; Test braces and parentheses around an entry:
+(ert-deftest parsebib-test-read-entry-parentheses ()
+  ;; Braces.
+  (should (equal
+           (with-temp-buffer
+             (insert "@book{Alexiadou:Haegeman:Stavrou2007,\n"
+                     "	year = {2007},\n"
+                     "	publisher = MGrt,\n"
+                     "	title = {Noun Phrase in the Generative Perspective},\n"
+                     "	author = {Alexiadou, Artemis and Haegeman, Liliane and Stavrou, Melita},\n"
+                     "	timestamp = {2013-09-25 12:00:00 (CET)},\n"
+                     "	file = {a/Alexiadou_Haegeman_Stavrou2007.pdf}}\n")
+             (goto-char (point-min))
+             (let ((results (parsebib-read-entry)))
+               (alist-get "=key=" results nil nil #'equal)))
+           "Alexiadou:Haegeman:Stavrou2007"))
+  ;; Parentheses.
+  (should (equal
+           (with-temp-buffer
+             (insert "@book(Alexiadou:Haegeman:Stavrou2007,\n"
+                     "	year = {2007},\n"
+                     "	publisher = MGrt,\n"
+                     "	title = {Noun Phrase in the Generative Perspective},\n"
+                     "	author = {Alexiadou, Artemis and Haegeman, Liliane and Stavrou, Melita},\n"
+                     "	timestamp = {2013-09-25 12:00:00 (CET)},\n"
+                     "	file = {a/Alexiadou_Haegeman_Stavrou2007.pdf})\n")
+             (goto-char (point-min))
+             (let ((results (parsebib-read-entry)))
+               (alist-get "=key=" results nil nil #'equal)))
+           "Alexiadou:Haegeman:Stavrou2007"))
+  ;; Mismatched should error.
+  (should-error (with-temp-buffer
+                  (insert "@book(Alexiadou:Haegeman:Stavrou2007,\n"
+                          "	year = {2007},\n"
+                          "	publisher = MGrt,\n"
+                          "	title = {Noun Phrase in the Generative Perspective},\n"
+                          "	author = {Alexiadou, Artemis and Haegeman, Liliane and Stavrou, Melita},\n"
+                          "	timestamp = {2013-09-25 12:00:00 (CET)},\n"
+                          "	file = {a/Alexiadou_Haegeman_Stavrou2007.pdf}}\n")
+                  (goto-char (point-min))
+                  (let ((results (parsebib-read-entry)))
+                    (alist-get "=key=" results nil nil #'equal)))
+                :type 'parsebib-error))
+
+;; Test unbalanced parentheses in field values.
+(ert-deftest parsebib-test-unbalanced-parentheses-in-fields ()
+  ;; (ASCII) LEFT PARENTHESIS
+  (should (equal (with-temp-buffer
+                   (insert
+                    "@article{Title,\n"
+                    "    title = {{Title}},\n"
+                    "    author = {Author},\n"
+                    "    year = {1970},\n"
+                    "    journal = {Journal},\n"
+                    "    abstract = {(}\n"
+                    "}\n")
+                   (goto-char (point-min))
+                   (let ((results (parsebib-read-entry)))
+                     (alist-get "abstract" results nil nil #'equal)))
+                 "{(}"))
+  ;; (ASCII) RIGHT PARENTHESIS
+  (should (equal (with-temp-buffer
+                   (insert
+                    "@article{Title,\n"
+                    "    title = {{Title}},\n"
+                    "    author = {Author},\n"
+                    "    year = {1970},\n"
+                    "    journal = {Journal},\n"
+                    "    abstract = {)}\n"
+                    "}\n")
+                   (goto-char (point-min))
+                   (let ((results (parsebib-read-entry)))
+                     (alist-get "abstract" results nil nil #'equal)))
+                 "{)}"))
+  ;; (ASCII) LEFT and RIGHT CURLY BRACKET with double quotes as delimiters.
+  (should (equal (with-temp-buffer
+                   (insert
+                    "@article{Title,\n"
+                    "    title = {{Title}},\n"
+                    "    author = {Author},\n"
+                    "    year = {1970},\n"
+                    "    journal = \"Journal{\",\n"
+                    "    abstract = \"Abstract}\"\n"
+                    "}\n")
+                   (goto-char (point-min))
+                   (let ((results (parsebib-read-entry)))
+                     (cons (alist-get "journal" results nil nil #'equal)
+                           (alist-get "abstract" results nil nil #'equal))))
+                 (cons "\"Journal{\"" "\"Abstract}\"")))
+  ;; (CJK) FULLWIDTH LEFT PARENTHESIS
+  (should (equal (with-temp-buffer
+                   (insert
+                    "@article{Title,\n"
+                    "    title = {{Title}},\n"
+                    "    author = {Author},\n"
+                    "    year = {1970},\n"
+                    "    journal = {Journal},\n"
+                    "    abstract = {（}\n"
+                    "}\n")
+                   (goto-char (point-min))
+                   (let ((results (parsebib-read-entry)))
+                     (alist-get "abstract" results nil nil #'equal)))
+                 "{（}"))
+  ;; (CJK) FULLWIDTH LEFT PARENTHESIS
+  (should (equal (with-temp-buffer
+                   (insert
+                    "@article{Title,\n"
+                    "    title = {{Title}},\n"
+                    "    author = {Author},\n"
+                    "    year = {1970},\n"
+                    "    journal = {Journal},\n"
+                    "    abstract = {）}\n"
+                    "}\n")
+                   (goto-char (point-min))
+                   (let ((results (parsebib-read-entry)))
+                     (alist-get "abstract" results nil nil #'equal)))
+                 "{）}"))
+  ;; (CJK) FULLWIDTH LEFT and RIGHT PARENTHESIS with double quotes as delimiters.
+  (should (equal (with-temp-buffer
+                   (insert
+                    "@article{Title,\n"
+                    "    title = {{Title}},\n"
+                    "    author = {Author},\n"
+                    "    year = {1970},\n"
+                    "    journal = \"Journal（\",\n"
+                    "    abstract = \"Abstract）\"\n"
+                    "}\n")
+                   (goto-char (point-min))
+                   (let ((results (parsebib-read-entry)))
+                     (cons (alist-get "journal" results nil nil #'equal)
+                           (alist-get "abstract" results nil nil #'equal))))
+                 (cons "\"Journal（\"" "\"Abstract）\""))))
+
 ;;; parsebib-test.el ends here
